# value_combo_row.py
#
# Copyright 2025 Mitchell Winkelman (Heliguy)
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# SPDX-License-Identifier: GPL-3.0-or-later


from typing import Any

from gi.repository import Gtk

from refine.widgets.combo_row import RefineComboRow
from refine.compositions import RefineCompositions

import logging


@Gtk.Template(resource_path="/page/tesk/Refine/widgets/value-combo-row.ui")
class RefineValueComboRow(RefineComboRow):
    """
    An `Adw.ComboRow` implementing the properties needed to
    work with enumerated properties in `Gio.Settings`.
    """

    __gtype_name__ = "RefineValueComboRow"

    revert_button = Gtk.Template.Child()  # type: ignore [misc]
    revealer = Gtk.Template.Child()  # type: ignore [misc]

    def __init__(self, **kwargs: Any) -> None:
        super().__init__(**kwargs)

    @RefineCompositions.key
    def key(self, key: str):
        try:
            if not self.schema.has_key(key):
                raise TypeError

        except (IndexError, TypeError):
            self.is_valid_setting = False
            logging.warning(f"Key “{key}” does not exist. Skipping…")
            return

        try:
            enum_tuple: tuple[str, list[str]] = (
                self.schema.get_key(key).get_range().unpack()
            )
            if len(enum_tuple) < 2 or enum_tuple[0] != "enum":
                raise TypeError

            enum_values: list[str] = sorted(
                enum_tuple[1], key=lambda item: item.lower()
            )
            model = Gtk.StringList.new(enum_values)
            string: str = self.settings.get_string(key)
            self.set_model(model)
            self.select_item(model, string)

        except TypeError:
            self.is_valid_setting = False
            logging.warning(f"Key “{key}” is not an enumerated property. Skipping…")
            return

        self.set_is_default(key, model)
        self.connect("notify::selected", self.selected_callback)
